@@ -16,10 +16,13 @@ pub struct Include {
 }
 
 #[derive(Debug)]
-pub struct LocalVariable {
+pub struct StaticVariable {
     pub captured: String,
-    pub name: String,
+    pub name_expr: String, // like "array[10]"
+    pub name: String,      // like "array"
     pub dtype: String,
+    pub is_local: bool,    // static variable declared within function
+    pub func_name: String, // function name where local variable is declared
 }
 
 #[derive(Debug, Clone)]
@@ -41,7 +44,7 @@ pub struct NestedCall {
 #[derive(Debug)]
 pub struct Parser {
     incs: Vec<Include>,
-    local_vars: Vec<LocalVariable>,
+    static_vars: Vec<StaticVariable>,
     fncs: Vec<Function>,
     ncls: Vec<NestedCall>,
 }
@@ -55,7 +58,7 @@ impl Parser {
         let ncls = get_ncls(&code, &fncs);
         Self {
             incs: get_incs(&code),
-            local_vars: get_local_vars(&code),
+            static_vars: get_static_vars(&code, &fncs),
             fncs: fncs.clone(),
             ncls: ncls,
         }
@@ -70,11 +73,13 @@ impl Parser {
 /// - **@incs@** : the list of inclusion statement such as `#include <stdio.h>`
 ///     - **@captured@** : the captured raw string
 /// - **@end-incs@** : the end of **incs** block
-/// - **@local-vars@** : the list of **static** variables
+/// - **@static-vars@** or **@static-global-vars@** or **@static-local-vars@**: the list of **static** variables
 ///     - **@captured@** : the captured raw string
 ///     - **@name@** : variable name
+///     - **@name-expr@** : variable name including brackets when array data
 ///     - **@dtype@** : variable data type
-/// - **@end-local-vars@** : the end of **local-var** bolck
+///     - **@func-name@** : function name only for **static-local-vars**
+/// - **@end-static-vars@** or **@end-static-global-vars@** or **@end-static-local-vars@**: the end of **static-vars** bolck
 /// - **@fncs@** or **@fncs0@** : the list of all the functions
 ///     - **@captured@** : the captured raw string
 ///     - **@name@** : the function name
@@ -144,23 +149,65 @@ pub fn generate<'a>(parser: &'a Parser, template: &'a str, sourcename: &'a str)
             output = re.replace(&output, tmpstr.as_str()).into_owned();
         }
     }
-    if template.contains("@local-vars@") {
-        let re = Regex::new(r"@local-vars@(?P<fmt>[\S\s]*)@end-local-vars@").unwrap();
+    if template.contains("@static-vars@") {
+        let re = Regex::new(r"@static-vars@(?P<fmt>[\S\s]*)@end-static-vars@").unwrap();
         for cap in re.captures_iter(template) {
             let mut tmpstr = String::new();
-            for entry in &parser.local_vars {
+            for entry in &parser.static_vars {
                 let fmtstr = cap
                     .name("fmt")
                     .unwrap()
                     .as_str()
                     .replace("@captured@", &entry.captured)
                     .replace("@name@", &entry.name)
+                    .replace("@name-expr@", &entry.name_expr)
                     .replace("@dtype@", &entry.dtype);
                 tmpstr.push_str(&fmtstr);
             }
             output = re.replace(&output, tmpstr.as_str()).into_owned();
         }
     }
+    if template.contains("@static-global-vars@") {
+        let re = Regex::new(r"@static-global-vars@(?P<fmt>[\S\s]*)@end-static-global-vars@").unwrap();
+        for cap in re.captures_iter(template) {
+            let mut tmpstr = String::new();
+            for entry in &parser.static_vars {
+                if !entry.is_local {
+                    let fmtstr = cap
+                        .name("fmt")
+                        .unwrap()
+                        .as_str()
+                        .replace("@captured@", &entry.captured)
+                        .replace("@name@", &entry.name)
+                        .replace("@name-expr@", &entry.name_expr)
+                        .replace("@dtype@", &entry.dtype);
+                    tmpstr.push_str(&fmtstr);
+                }
+            }
+            output = re.replace(&output, tmpstr.as_str()).into_owned();
+        }
+    }
+    if template.contains("@static-local-vars@") {
+        let re = Regex::new(r"@static-local-vars@(?P<fmt>[\S\s]*)@end-static-local-vars@").unwrap();
+        for cap in re.captures_iter(template) {
+            let mut tmpstr = String::new();
+            for entry in &parser.static_vars {
+                if entry.is_local {
+                    let fmtstr = cap
+                        .name("fmt")
+                        .unwrap()
+                        .as_str()
+                        .replace("@captured@", &entry.captured)
+                        .replace("@name@", &entry.name)
+                        .replace("@name-expr@", &entry.name_expr)
+                        .replace("@func-name@", &entry.func_name)
+                        .replace("@dtype@", &entry.dtype);
+                    tmpstr.push_str(&fmtstr);
+                }
+            }
+            output = re.replace(&output, tmpstr.as_str()).into_owned();
+        }
+    }
     let fncs_tags = vec!["fncs", "fncs0"];
     for tag in fncs_tags {
         let regstr = format!("@{}@{}@end-{}@", tag, r"(?P<fmt>[\S\s]*)", tag);
@@ -299,13 +346,14 @@ fn get_incs<'a>(code: &'a str) -> Vec<Include> {
     result
 }
 
-/// list of local variables from C source code
+/// list of static variables from C source code
 ///
-fn get_local_vars<'a>(code: &'a str) -> Vec<LocalVariable> {
+fn get_static_vars<'a>(code: &'a str, fncs: &Vec<Function>) -> Vec<StaticVariable> {
     let mut result = vec![];
     let re = Regex::new(r"(?i)(static)\s+(?P<captured>[^\(\{;=]+)[;=]").unwrap();
     for cap in re.captures_iter(code) {
-        let name = cap
+        let captured = cap.name("captured").unwrap().as_str().trim().to_string();
+        let name_expr = cap
             .name("captured")
             .unwrap()
             .as_str()
@@ -316,19 +364,40 @@ fn get_local_vars<'a>(code: &'a str) -> Vec<LocalVariable> {
             .unwrap()
             .trim_matches('*')
             .to_string();
+        let name = if let Some(exprs) = name_expr.split_once('[') {
+            exprs.0.to_string()
+        } else {
+            name_expr.clone()
+        };
+        let mut is_local = false;
+        let mut func_name = String::from("");
         let data_type = cap
             .name("captured")
             .unwrap()
             .as_str()
             .trim()
-            .strip_suffix(&name)
+            .strip_suffix(&name_expr)
             .unwrap()
             .trim()
             .to_string();
-        result.push(LocalVariable {
-            captured: cap.name("captured").unwrap().as_str().trim().to_string(),
+        for func in fncs {
+            if let Some(pos) = code.find(func.captured.as_str()) {
+                let start = pos + code.get(pos..).unwrap().find('{').unwrap() + 1;
+                let stop = find_end_of_func(code, start);
+                let body = code.get(start..stop).unwrap();
+                if body.contains(captured.as_str()) {
+                    is_local = true;
+                    func_name = func.name.to_string();
+                }
+            }
+        }
+        result.push(StaticVariable {
+            captured: captured,
+            name_expr: name_expr,
             name: name,
             dtype: data_type,
+            is_local: is_local,
+            func_name: func_name,
         });
     }
     result
@@ -411,30 +480,33 @@ fn get_fncs<'a>(code: &'a str) -> Vec<Function> {
     result
 }
 
+/// find end of func
+///
+fn find_end_of_func<'a>(code: &'a str, start: usize) -> usize {
+    let mut stop = start;
+    let mut open = 1;
+    for (i, c) in code.get(start..).unwrap().chars().enumerate() {
+        if c == '}' {
+            open -= 1;
+        } else if c == '{' {
+            open += 1;
+        }
+        if open == 0 {
+            stop += i;
+            break;
+        }
+    }
+    stop
+}
+
 /// list of ncls in C source
 ///
 fn get_ncls<'a>(code: &'a str, fncs: &Vec<Function>) -> Vec<NestedCall> {
     let mut result = vec![];
     for caller in fncs {
-        let find_end_of_func = |start: usize| -> usize {
-            let mut stop = start;
-            let mut open = 1;
-            for (i, c) in code.get(start..).unwrap().chars().enumerate() {
-                if c == '}' {
-                    open -= 1;
-                } else if c == '{' {
-                    open += 1;
-                }
-                if open == 0 {
-                    stop += i;
-                    break;
-                }
-            }
-            stop
-        };
         if let Some(pos) = code.find(caller.captured.as_str()) {
             let start = pos + code.get(pos..).unwrap().find('{').unwrap() + 1;
-            let stop = find_end_of_func(start);
+            let stop = find_end_of_func(code, start);
             let body = code.get(start..stop).unwrap();
             for callee in fncs {
                 let call_str = format!("{}(", callee.name);
@@ -461,7 +533,7 @@ mod tests {
 #include \"test.h\"
 
 int global_var = 2;
-static char local_var;
+static char static_var;
 
 // test-comment1
 /* test-comment2 */
@@ -487,6 +559,7 @@ void main()
 static inline char local_function(int a, 
     int*b )
 {
+    static int local_var[10];
     return (char)a;
 }
 ";
@@ -512,10 +585,17 @@ static inline char local_function(int a,
     }
 
     #[test]
-    fn test_get_local_vars() {
-        let list_local_vars = get_local_vars(TEST_CODE);
-        assert_eq!(list_local_vars[0].name, "local_var");
-        assert_eq!(list_local_vars[0].dtype, "char");
+    fn test_get_static_vars() {
+        let list_fncs = get_fncs(TEST_CODE);
+        let list_static_vars = get_static_vars(TEST_CODE, &list_fncs);
+        assert_eq!(list_static_vars[0].name, "static_var");
+        assert_eq!(list_static_vars[0].dtype, "char");
+        assert_eq!(list_static_vars[0].is_local, false);
+        assert_eq!(list_static_vars[1].name, "local_var");
+        assert_eq!(list_static_vars[1].name_expr, "local_var[10]");
+        assert_eq!(list_static_vars[1].dtype, "int");
+        assert_eq!(list_static_vars[1].is_local, true);
+        assert_eq!(list_static_vars[1].func_name, "local_function");
     }
 
     #[test]
@@ -566,23 +646,41 @@ static inline char local_function(int a,
     }
 
     #[test]
-    fn test_generate_localvars() {
+    fn test_generate_static_vars() {
         let sourcename = "test";
         let code = "\
-static int a;
+static int a[10];
 int b;
 static char *c;
+void func1(void)
+{
+    static int local_var;
+}
 ";
         let temp = "\
 // static variables
-@local-vars@@dtype@ @name@;
-@end-local-vars@
+@static-vars@@dtype@ @name-expr@;
+@end-static-vars@
+// static global variables
+@static-global-vars@@dtype@ @name@;
+@end-static-global-vars@
+// static local variables
+@static-local-vars@@dtype@ @name@;
+@end-static-local-vars@
 ";
         let expected = "\
 // static variables
+int a[10];
+char * c;
+int local_var;
+
+// static global variables
 int a;
 char * c;
 
+// static local variables
+int local_var;
+
 ";
         let parser = Parser::parse(code);
         let generated = generate(&parser, temp, sourcename);
